/*
Autor: Marius M.
Kontakt: www.montisblog.de
Funktion: Dieses Programm erstellt einen Webserver mit einem Arduino und einem Ethernet Shield.
Bevor das Programm auf deinen Arduino geladen werden kann, muss noch die MAC-Adresse von deinem
Ethernet Shield eingetragen werden. Auch die IP-Adresse musst du evtl. im Programmcode anpassen.
Der Webserver reagiert auf Anfragen eines Webbrowsers auf einem Computer. Wenn ein Button
gedrückt wird, schaltet dieses Programm eine Funksteckdose mit einer Frequenz von 433,92 Mhz.
Für genauere Informationen lies dir den Artikel zu den Funksteckdosen auf meinem Blog durch.

*/

#include <SPI.h>
#include <Ethernet.h>
#include <TextFinder.h>
#include <SD.h>


byte mac[] = { 
  0x90, 0xA2, 0xDA, 0x0E, 0xC8, 0xC0 }; // MAC-Adresse des Ethernet-Shield
byte ip[]  = { 
  192, 168, 2, 125 };                   // IP zum aufrufen des Webservers
byte sdPin = 4;       // Pin der SD-Karte

int aktBit;
int rfpin = 26;

EthernetServer server(80);  // Serverport (80 ist der Standard-http-Port)

File webFile;


const int OnCodes [16] [12] = { 
  {
    0,0,1,1,1,1,1,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,1,1,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,1,0,1,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,0,1,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,1,1,0,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,1,0,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,1,0,0,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,0,0,1,1,0,0,0                  }
  ,
  {
    0,0,1,1,1,1,1,0,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,1,1,0,1,0,0,0                  }
  ,
  {
    0,0,1,1,1,0,1,0,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,0,1,0,1,0,0,0                  }
  ,
  {
    0,0,1,1,1,1,0,0,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,1,0,0,1,0,0,0                  }
  ,
  {
    0,0,1,1,1,0,0,0,1,0,0,0                  }
  ,
  {
    0,0,1,1,0,0,0,0,1,0,0,0                  }
};

//Eine Array mit den 16 Codes, mit der die Funksteckdosen ausgeschaltet werden
const int OffCodes [16] [12] = {
  {
    0,0,1,1,1,1,1,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,1,1,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,1,0,1,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,0,1,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,1,1,0,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,1,0,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,1,0,0,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,0,0,1,1,0,0,1                  }
  ,
  {
    0,0,1,1,1,1,1,0,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,1,1,0,1,0,0,1                  }
  ,
  {
    0,0,1,1,1,0,1,0,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,0,1,0,1,0,0,1                  }
  ,
  {
    0,0,1,1,1,1,0,0,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,1,0,0,1,0,0,1                  }
  ,
  {
    0,0,1,1,1,0,0,0,1,0,0,1                  }
  ,
  {
    0,0,1,1,0,0,0,0,1,0,0,1                  }
};



void setup()
{  

  Serial.begin(9600);
  Ethernet.begin(mac, ip); // Client starten
  server.begin();          // Server starten
  Serial.println("Arduino WLAN-Steckdosen-Server");

  Serial.println("Suche nach SD-Karte...");
  if (!SD.begin(sdPin)) 
  {
    Serial.println(" - Keine SD-Karte vorhanden!");
    return;
  }
  Serial.println(" - SD-Karte gefunden und einsatzbereit!");

  if (!SD.exists("index.html")) 
  {
    Serial.println(" - Die benötigte Datei (index.html) wurde nicht gefunden!");
    return;
  }
  Serial.println(" - Die benötigte Datei (index.html) wurde gefunden!");

  Serial.println();
  Serial.println("Server ist einsatzbereit!");
}

void loop()
{
  EthernetClient client = server.available(); // Auf Anfrage warten

  if(client)
  {
    //Auf die Buttons auf der Webseite reagieren

    TextFinder finder(client);

    if(finder.find("GET"))
    {
      while(finder.findUntil("Ger", "\n\r"))
      {
        char typ = client.read();
        int  webGeraet = finder.getValue();
        int  webZustand = finder.getValue();

        if(typ == 'G')
        {
          Schalten(webGeraet, webZustand);
          Serial.println("Gerät " + webGeraet + " wurde ");
          if (webZustand == 1) {
            Serial.print("eingeschaltet!"); 
          }
          
          if (webZustand == 0) {
            Serial.print("eingeschaltet!"); 
          }
        }

      }
    }


    //Webseite anzeigen
    boolean Anfrage = true;       // eine HTTP-Anfrage endet mit einer Leerzeile und einer neuen Zeile

      while (client.connected())  
    {
      if (client.available())    //Wenn der Arduino von einem Computer kontaktiert wurde.      
      {
        char c = client.read();     // Anfrage vom Webbrowser auslesen
        if (c == '\n' && Anfrage)   // wenn neue Zeile und Leerzeile empfangen
        {                           // Standard HTTP Header senden
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close");
          client.println();

          // Website von SD-Karte laden
          webFile = SD.open("index.html");  // Datei öffnen
          if (webFile)
          {
            while(webFile.available())
            {
              client.write(webFile.read()); // Website an Client schicken
            }
            webFile.close();
          }
          break;
        }
        if (c == '\n') 
        {
          Anfrage = true;
        } 
        else if (c != '\r') 
        {
          Anfrage = false;
        }
      }
    }
    delay(1);
    client.stop();
  }
}


void Schalten (int sltGerat, int sltZustand){
  //Methode um zu definieren, welches Gerät in welchen Zustand geschaltet werden soll.
  //Aufruf: Schalten(Gerätenummer (1 - 16), Zustand (1, 0);
  sltGerat = sltGerat - 1;

  if (sltZustand == 1){
    for (int j = 0; j < 7; j++){
      for (int i = 0; i < 12; i++){
        aktBit = OnCodes[sltGerat][i];
        sendBit(aktBit);
      }
      sendBit(2);
    }
  }
  if (sltZustand == 0){
    for (int j = 0; j < 7; j++){
      for (int i = 0; i < 12; i++){
        aktBit = OffCodes[sltGerat][i];
        sendBit(aktBit);
      }
      sendBit(2);
    }
  }

}


void sendBit(int z) { //Diese Funktion soll 0,1 oder 2 (sync) senden können.
  if (z == 0)
  { //Der Code für '0'
    digitalWrite(rfpin,HIGH);
    wait(1); //Warte ein mal 350 Mikrosekunden (kurzer Ausschalg)
    digitalWrite(rfpin,LOW);
    wait(3); //Warte drei mal 350 Mikrosekunden (lange Pause)
    digitalWrite(rfpin,HIGH);
    wait(3); //Warte drei mal 350 Mikrosekunden (langer Ausschalg)
    digitalWrite(rfpin,LOW);
    wait(1); //Warte ein mal 350 Mikrosekunden (kurze Pause)
  } 
  else
    if (z == 1)
    { //Der Code für '1'
      digitalWrite(rfpin,HIGH);
      wait(1); //Warte ein mal 350 Mikrosekunden (kurzer Ausschalg)
      digitalWrite(rfpin,LOW);
      wait(3); //Warte drei mal 350 Mikrosekunden (lange Pause)
      digitalWrite(rfpin,HIGH);
      wait(1); //Warte ein mal 350 Mikrosekunden (kurzer Ausschalg)
      digitalWrite(rfpin,LOW);
      wait(3); //Warte drei mal 350 Mikrosekunden (lange Pause)
    } 
    else
      if (z == 2)
      { //Der Code für das Synchronisationssignal
        digitalWrite(rfpin,HIGH);
        wait(1); //Warte ein mal 350 Mikrosekunden (kurzer Ausschalg)
        digitalWrite(rfpin,LOW);
        wait(31); //Warte 31 mal 350 Mikrosekunden (sehr lange Pause)
      }

}


void wait(int x) {
  //Methode um eine bestimmte Zeit nach jedem Signal zu warten.
  delayMicroseconds(x*350); //x*350 Mikrosekunden zwischen den Signalen warten
}


